# -*- coding: utf-8 -*-
# Author: allen
# Copyright (c) 2022 RuiCare All rights reserved.
import random
import numpy as np
import xlrd
from keras.utils import Sequence


class Generator(Sequence):

    def __init__(self, excel_path, batch_size=2, sheet_index=0, begin_row=None, end_row=None):
        self.excel_path = excel_path
        self.batch_size = batch_size
        self.excel = xlrd.open_workbook(self.excel_path)
        self.sheet = self.excel.sheets()[sheet_index]
        self.rows = self.sheet.nrows
        self.cols = self.sheet.ncols
        self.begin_row = 2 if begin_row is None else begin_row
        self.end_row = self.rows - 1 if end_row is None else end_row
        self.scope_rows = self.end_row - self.begin_row + 1
        self.steps_per_epoch = self.__len__()

    def __len__(self):
        return self.scope_rows // self.batch_size

    def __getitem__(self, i):
        batch_x = np.zeros(shape=(self.batch_size, 42), dtype=np.float16)
        batch_y = np.zeros(shape=(self.batch_size, 3), dtype=np.float16)

        row_index_list = random.sample(range(self.begin_row, self.end_row + 1), self.batch_size)

        for i, row_index in enumerate(row_index_list):
            # print(row_index)
            values = np.asarray(self.sheet.row_values(row_index), dtype=np.float16)
            # values[values < 0] = 0
            batch_x[i, :] = values[1:]
            batch_y[i, int(values[0])] = 1

        return batch_x, batch_y

    def get_all_items(self):

        batch_x = np.zeros(shape=(self.scope_rows, 42), dtype=np.float16)
        batch_y = np.zeros(shape=(self.scope_rows, 3), dtype=np.float16)

        for i, row_index in enumerate(range(self.begin_row, self.end_row + 1)):
            # print(row_index)
            values = np.asarray(self.sheet.row_values(row_index), dtype=np.float16)
            # values[values < 0] = 0
            batch_x[i, :] = values[1:]
            batch_y[i, int(values[0])] = 1

        return batch_x, batch_y


if __name__ == '__main__':
    gen = Generator(excel_path='/home/allen/Desktop/cervix.xlsx', batch_size=2, begin_row=5, end_row=10)
    batch_x, batch_y = gen.__getitem__(0)
    # print(batch_x)
    print(batch_y)

    # batch_x, batch_y = gen.get_all_items()
    # print(batch_y)
