# -*- coding: utf-8 -*-
# Author: allen
# Copyright (c) 2020 RuiCare All rights reserved.
import os
import codecs
import shutil


def remove_all_with_self(dir):
    if os.path.exists(dir): shutil.rmtree(dir)


def remove_all_without_self(dir):
    remove_all_with_self(dir)
    if not os.path.exists(dir):
        os.makedirs(dir)


def write(file_path, contents):
    f = codecs.open(file_path, 'w', encoding='UTF-8')
    f.write(contents)
    f.close()


def load(file_path):
    f = codecs.open(file_path, 'r', encoding='UTF-8')
    content = f.read()
    f.close()
    return content


def list_file_path(dir, type='', recursion=True, sort_by_mtime=False):
    all_file_path = []
    for file in os.listdir(dir):
        file_path = os.path.join(dir, file)
        if not os.path.isdir(file_path):
            # 过滤mac上.DS_Store文件
            if file.startswith('.') or not file.endswith(type): continue
            all_file_path.append(file_path)
        else:
            if recursion: all_file_path.extend(list_file_path(file_path))
    if sort_by_mtime:
        all_file_path = sorted(all_file_path, key=lambda x: os.path.getmtime(x))
    else:
        all_file_path.sort()
    return all_file_path


def list_dir(dir):
    dirs = [os.path.join(dir, d) for d in os.listdir(dir) if os.path.isdir(os.path.join(dir, d))]
    dirs.sort()
    return dirs


def list_all_file_path(dir, type=''):
    all_file_path = []
    for file in os.listdir(dir):
        file_path = os.path.join(dir, file)
        if not os.path.isdir(file_path):
            if not file.endswith(type): continue
            all_file_path.append(file_path)
        else:
            all_file_path.extend(list_all_file_path(file_path))
    all_file_path.sort()
    return all_file_path


def list_dir_count(root_dir):
    if not os.path.exists(root_dir): return 0
    object_names = os.listdir(root_dir)
    count = 0
    for objectName in object_names:
        object_path = os.path.join(root_dir, objectName)
        if os.path.isdir(object_path): count += 1
    return count


def list_dir_name(root_dir):
    if not os.path.exists(root_dir): return []
    object_names = os.listdir(root_dir)
    dir_names = []
    for objectName in object_names:
        object_path = os.path.join(root_dir, objectName)
        if os.path.isdir(object_path):
            dir_names.append(objectName)
    return dir_names


def copy_file(source_file_path, dest_file_path):
    check_and_create_dir_for_new_file(dest_file_path)
    shutil.copy(source_file_path, dest_file_path)


def move_file(source_file_path, dest_file_path):
    check_and_create_dir_for_new_file(dest_file_path)
    shutil.move(source_file_path, dest_file_path)


def delete_empty_dir_without_self(root_dir):
    delete_empty_dir_with_self(root_dir, root_dir)


def delete_empty_dir_with_self(root_dir, *except_dirs):
    if os.path.isdir(root_dir):
        for d in os.listdir(root_dir):
            delete_empty_dir_with_self(os.path.join(root_dir, d))
    if not os.listdir(root_dir):
        if not root_dir in except_dirs:
            os.rmdir(root_dir)


def check_and_clear_dir(*dirs):
    for dir in dirs:
        if os.path.exists(dir):
            remove_all_without_self(dir)
        else:
            os.makedirs(dir)


def check_and_create_dir(*dirs):
    for dir in dirs:
        if not os.path.exists(dir): os.makedirs(dir)


def check_and_create_dir_for_new_file(file_path):
    dir = os.path.dirname(file_path)
    if not os.path.exists(dir): os.makedirs(dir)
